@@ -98,9 +98,9 @@ static void battery_submit (const char *plugin_instance, const char *type, doubl
 	vl.values = values;
 	vl.values_len = 1;
 	vl.time = time (NULL);
-	strcpy (vl.host, hostname_g);
-	strcpy (vl.plugin, "battery");
-	strcpy (vl.plugin_instance, plugin_instance);
+	sstrncpy (vl.host, hostname_g, sizeof (vl.host));
+	sstrncpy (vl.plugin, "battery", sizeof (vl.plugin));
+	sstrncpy (vl.plugin_instance, plugin_instance, sizeof (vl.plugin_instance));
 
 	plugin_dispatch_values (type, &vl);
 } /* void battery_submit */
