@@ -27,9 +27,15 @@
 #include <sys/socket.h>
 #include <netdb.h>
 
+#include <pthread.h>
+
 #include "plugin.h"
 #include "configfile.h"
 
+#if HAVE_STATGRAB_H
+# include <statgrab.h>
+#endif
+
 /*
  * Global variables
  */
@@ -41,16 +47,37 @@ kstat_ctl_t *kc;
 
 static int loop = 0;
 
-static void sigIntHandler (int signal)
+static void *do_flush (void *arg)
+{
+	INFO ("Flushing all data.");
+	plugin_flush_all (-1);
+	INFO ("Finished flushing all data.");
+	pthread_exit (NULL);
+	return NULL;
+}
+
+static void sig_int_handler (int signal)
 {
 	loop++;
 }
 
-static void sigTermHandler (int signal)
+static void sig_term_handler (int signal)
 {
 	loop++;
 }
 
+static void sig_usr1_handler (int signal)
+{
+	pthread_t      thread;
+	pthread_attr_t attr;
+
+	/* flushing the data might take a while,
+	 * so it should be done asynchronously */
+	pthread_attr_init (&attr);
+	pthread_attr_setdetachstate (&attr, PTHREAD_CREATE_DETACHED);
+	pthread_create (&thread, &attr, do_flush, NULL);
+}
+
 static int init_hostname (void)
 {
 	const char *str;
@@ -91,7 +118,7 @@ static int init_hostname (void)
 				"\"FQDNLookup\" option, but I cannot resolve "
 				"my hostname to a fully qualified domain "
 				"name. Please fix you network "
-				"configuration.");
+				"configuration.", hostname_g);
 		return (-1);
 	}
 
@@ -227,11 +254,13 @@ static void exit_usage (void)
 			"  General:\n"
 			"    -C <file>       Configuration file.\n"
 			"                    Default: "CONFIGFILE"\n"
+			"    -t              Test config and exit.\n"
 			"    -P <file>       PID-file.\n"
 			"                    Default: "PIDFILE"\n"
 #if COLLECT_DAEMON
 			"    -f              Don't fork to the background.\n"
 #endif
+			"    -h              Display help (this message)\n"
 			"\nBuiltin defaults:\n"
 			"  Config-File       "CONFIGFILE"\n"
 			"  PID-File          "PIDFILE"\n"
@@ -365,13 +394,15 @@ static int pidfile_remove (void)
 
 int main (int argc, char **argv)
 {
-	struct sigaction sigIntAction;
-	struct sigaction sigTermAction;
+	struct sigaction sig_int_action;
+	struct sigaction sig_term_action;
+	struct sigaction sig_usr1_action;
+	struct sigaction sig_pipe_action;
 	char *configfile = CONFIGFILE;
 	int test_config  = 0;
 	const char *basedir;
 #if COLLECT_DAEMON
-	struct sigaction sigChldAction;
+	struct sigaction sig_chld_action;
 	pid_t pid;
 	int daemonize    = 1;
 #endif
@@ -456,9 +487,9 @@ int main (int argc, char **argv)
 	/*
 	 * fork off child
 	 */
-	memset (&sigChldAction, '\0', sizeof (sigChldAction));
-	sigChldAction.sa_handler = SIG_IGN;
-	sigaction (SIGCHLD, &sigChldAction, NULL);
+	memset (&sig_chld_action, '\0', sizeof (sig_chld_action));
+	sig_chld_action.sa_handler = SIG_IGN;
+	sigaction (SIGCHLD, &sig_chld_action, NULL);
 
 	if (daemonize)
 	{
@@ -508,16 +539,39 @@ int main (int argc, char **argv)
 	} /* if (daemonize) */
 #endif /* COLLECT_DAEMON */
 
+	memset (&sig_pipe_action, '\0', sizeof (sig_pipe_action));
+	sig_pipe_action.sa_handler = SIG_IGN;
+	sigaction (SIGPIPE, &sig_pipe_action, NULL);
+
 	/*
 	 * install signal handlers
 	 */
-	memset (&sigIntAction, '\0', sizeof (sigIntAction));
-	sigIntAction.sa_handler = sigIntHandler;
-	sigaction (SIGINT, &sigIntAction, NULL);
+	memset (&sig_int_action, '\0', sizeof (sig_int_action));
+	sig_int_action.sa_handler = sig_int_handler;
+	if (0 != sigaction (SIGINT, &sig_int_action, NULL)) {
+		char errbuf[1024];
+		ERROR ("Error: Failed to install a signal handler for signal INT: %s",
+				sstrerror (errno, errbuf, sizeof (errbuf)));
+		return (1);
+	}
 
-	memset (&sigTermAction, '\0', sizeof (sigTermAction));
-	sigTermAction.sa_handler = sigTermHandler;
-	sigaction (SIGTERM, &sigTermAction, NULL);
+	memset (&sig_term_action, '\0', sizeof (sig_term_action));
+	sig_term_action.sa_handler = sig_term_handler;
+	if (0 != sigaction (SIGTERM, &sig_term_action, NULL)) {
+		char errbuf[1024];
+		ERROR ("Error: Failed to install a signal handler for signal TERM: %s",
+				sstrerror (errno, errbuf, sizeof (errbuf)));
+		return (1);
+	}
+
+	memset (&sig_usr1_action, '\0', sizeof (sig_usr1_action));
+	sig_usr1_action.sa_handler = sig_usr1_handler;
+	if (0 != sigaction (SIGUSR1, &sig_usr1_action, NULL)) {
+		char errbuf[1024];
+		ERROR ("Error: Failed to install a signal handler for signal USR1: %s",
+				sstrerror (errno, errbuf, sizeof (errbuf)));
+		return (1);
+	}
 
 	/*
 	 * run the actual loops
