@@ -180,12 +180,12 @@ static void submit (int cpu_num, const char *type_instance, counter_t value)
 	vl.values = values;
 	vl.values_len = 1;
 	vl.time = time (NULL);
-	strcpy (vl.host, hostname_g);
-	strcpy (vl.plugin, "cpu");
+	sstrncpy (vl.host, hostname_g, sizeof (vl.host));
+	sstrncpy (vl.plugin, "cpu", sizeof (vl.plugin));
 	snprintf (vl.plugin_instance, sizeof (vl.type_instance),
 			"%i", cpu_num);
 	vl.plugin_instance[DATA_MAX_NAME_LEN - 1] = '\0';
-	strcpy (vl.type_instance, type_instance);
+	sstrncpy (vl.type_instance, type_instance, sizeof (vl.type_instance));
 
 	plugin_dispatch_values ("cpu", &vl);
 }
