@@ -216,8 +216,8 @@ static void multimeter_submit (double value)
 	vl.values = values;
 	vl.values_len = 1;
 	vl.time = time (NULL);
-	strcpy (vl.host, hostname_g);
-	strcpy (vl.plugin, "multimeter");
+	sstrncpy (vl.host, hostname_g, sizeof (vl.host));
+	sstrncpy (vl.plugin, "multimeter", sizeof (vl.plugin));
 
 	plugin_dispatch_values ("multimeter", &vl);
 }
